import Flutter
import MFSDK
import UIKit

public class MyfatoorahFlutterPlugin: NSObject, FlutterPlugin, MFSDKHelper {
    override init() {
        super.init()
        MFSettings.shared.delegate = self
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let messenger : FlutterBinaryMessenger = registrar.messenger()
        let api : MFSDKHelper & NSObjectProtocol = MyfatoorahFlutterPlugin.init()
        MFSDKHelperSetup.setUp(binaryMessenger: messenger, api: api);
                
        let eventChannel = FlutterEventChannel(name: MFHelper.MFEventChannelName, binaryMessenger: messenger)
        eventChannel.setStreamHandler(MFStreamHandler())
        
        let cardViewFactory = CardViewFactory(messenger: messenger)
        registrar.register(cardViewFactory, withId: MFHelper.cardViewModule)
        
        let applePayFactory = ApplePayFactory(messenger: messenger)
        registrar.register(applePayFactory, withId: MFHelper.applePayModule)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "getPlatformVersion":
            result("iOS " + UIDevice.current.systemVersion)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    //MARK: MFSDF Functions
    func loadConfig(apiKey: String,
                    country: String,
                    environment: String) {
        let country = MFHelper.toMFCountry(code: country)
        if let env = MFEnvironment(rawValue: environment.lowercased()) {
            MFSettings.shared.configure(token: apiKey, country: country, environment: env)
        }
    }
    
    func initiatePayment(initiatePaymentRequest: MFInitiatePaymentRequest,
                         lang: String,
                         completion: @escaping (Result<MFInitiatePaymentResponse, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        MFPaymentRequest.shared.initiatePayment(request: initiatePaymentRequest, apiLanguage: language) { response in
            switch response {
            case .success(let initiatePaymentResponse):
                completion(.success(initiatePaymentResponse))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func setUpActionBar( toolBarTitle: String,
                         toolBarTitleColor: Int64,
                         toolBarBackgroundColor: Int64,
                         isShowToolBar: Bool) {
        let titleColor = MFHelper.getColor(number: Int(toolBarTitleColor))
        let backgroundColor = MFHelper.getColor(number: Int(toolBarBackgroundColor))
        
        let them = MFTheme(navigationTintColor: titleColor, navigationBarTintColor: backgroundColor, navigationTitle: toolBarTitle as String, cancelButtonTitle: "cancel", showNavigationBar: isShowToolBar)
        MFSettings.shared.setTheme(theme: them)
    }
    
    func sendPayment(sendPaymentRequest: MFSendPaymentRequest,
                     lang: String,
                     completion: @escaping (Result<MFSendPaymentResponse, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        
        MFPaymentRequest.shared.sendPayment(request: sendPaymentRequest, apiLanguage: language) { response in
            switch response {
            case .success(let sendPaymentResponse):
                completion(.success(sendPaymentResponse))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func executePayment(executePaymentRequest: MFExecutePaymentRequest,
                        lang: String,
                        completion: @escaping (Result<MFPaymentStatusResponse, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        
        MFPaymentRequest.shared.executePayment(request: executePaymentRequest, apiLanguage: language) { (response, invoiceId) in
            switch response {
            case .success(let executePaymentResponse):
                completion(.success(executePaymentResponse))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func executeDirectPayment(directPaymentRequest: MFDirectPaymentContainer,
                              lang: String,
                              completion: @escaping (Result<MFDirectPaymentResponse, Error>) -> Void) {
        guard let executePayReq = directPaymentRequest.executePaymentRequest?.MFExecPayRequest else {
            completion(.failure(NSError(domain: "Error In Parsing Data: Incomplete Request", code: 400)))
            return
        }
        var card: MFCardInfo
        if let cardInfo = directPaymentRequest.card {
            card = MFCardInfo(cardNumber: cardInfo.number, cardExpiryMonth: cardInfo.expiryMonth, cardExpiryYear: cardInfo.expiryYear, cardHolderName: cardInfo.cardHolderName, cardSecurityCode: cardInfo.securityCode, saveToken: directPaymentRequest.saveToken ?? false)
            card.bypass = directPaymentRequest.bypass3DS ?? false
        } else if let token =  directPaymentRequest.token {
            card = MFCardInfo(cardToken: token)
            card.bypass = directPaymentRequest.bypass3DS ?? false
        } else {
            completion(.failure(NSError(domain: "Error In Parsing Data: Incomplete Request", code: 400)))
            return
        }
        MFPaymentRequest.shared.executeDirectPayment(request: executePayReq, cardInfo: card, apiLanguage: .english) { (response, invoiceId) in
            switch response {
            case .success(let directPaymentResponse):
                completion(.success(directPaymentResponse))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func getPaymentStatus(getPaymentStatusRequest: MFPaymentStatusRequest,
                          lang: String,
                          completion: @escaping (Result<MFPaymentStatusResponse, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        
        MFPaymentRequest.shared.getPaymentStatus(paymentStatus: getPaymentStatusRequest, apiLanguage: language) { response in
            switch response {
            case .success(let paymentStatusResponse):
                completion(.success(paymentStatusResponse))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func cancelToken( token: String,
                      lang: String,
                      completion: @escaping (Result<Bool, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        
        MFPaymentRequest.shared.cancelToken(token: token as String, apiLanguage: language) { result in
            switch result {
            case .success(let tokenResult):
                completion(.success(tokenResult))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func cancelRecurringPayment(recurringId: String,
                                lang: String,
                                completion: @escaping (Result<Bool, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        
        MFPaymentRequest.shared.cancelRecurringPayment(recurringId: recurringId, apiLanguage: language) { result in
            switch result {
            case .success(let recurringResult):
                completion(.success(recurringResult))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func initiateSession(initiateSessionRequest: MFInitiateSessionRequest,
                         completion: @escaping (Result<MFInitiateSessionResponse, Error>) -> Void) {
        MFPaymentRequest.shared.initiateSession(request: initiateSessionRequest, apiLanguage: .english) { response in
            switch response {
            case .success(let session):
                CardView.cardView.load(initiateSession: session) { bin in
                    MFStreamHandler.sendStream(data: bin)
                }
                completion(.success(session))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func pay(executePaymentRequest: MFExecutePaymentRequest,
             lang: String,
             completion: @escaping (Result<MFPaymentStatusResponse, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        CardView.cardView.pay(executePaymentRequest, language) { response, invoiceId in
            switch response {
            case .success(let paymentStatus):
                completion(.success(paymentStatus))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func validate(completion: @escaping (Result<String, Error>) -> Void) {
        CardView.cardView.validate() { result in
            switch result {
            case .success(let cardBrand):
                completion(.success(cardBrand))
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
    
    func applePayPayment(executePaymentRequest: MFExecutePaymentRequest,
                         lang: String,
                         completion: @escaping (Result<MFPaymentStatusResponse, Error>) -> Void) {
        let language = MFAPILanguage(rawValue: lang) ?? .english
        
        MFPaymentRequest.shared.initiateSession(apiLanguage: language) { response in
            switch response {
            case .success(let session):
                ApplePayButton.applePayButton.load(session, executePaymentRequest, language) { response, invoiceId in
                    switch response {
                    case .success(let executePaymentResponse):
                        completion(.success(executePaymentResponse))
                    case .failure(let error):
                        completion(.failure(error))
                    }
                }
            case .failure(let error):
                completion(.failure(error))
            }
        }
    }
}

extension MyfatoorahFlutterPlugin: MFPaymentDelegate {
    public func didInvoiceCreated(invoiceId: String) {
        MFStreamHandler.sendStream(data: invoiceId)
    }
}
